'use strict';

let dgram = require('dgram');
let client = dgram.createSocket("udp4");
let countApiServer = dgram.createSocket("udp4");
let co = require('co');
let BemZk = require('bem-zk');

let zkAPI;
let intervalId;
let hasInit = false;

let ArrearageList = {};
let bufferUserObj = {}; /* user call api counter */

let criticalData = {};
let countAPInormalData = {
  coutAPIPort: 12345,
  /* API udp server port number */
  CountSDPort: 12346,
  /* countSD port number */
  apiBufferUserCount: 200,
  /* max buffer user number call API, if get it, must send data out. */
  apiBufferTimer: 10,
  /* max buffer timer for call API, if get it, must send data out. unit is seond. */
  maxApiTypeNumber: 20
}; /* API type number, max is 10 so far. */

/*
 *----------------------------------------------------------------------
 *
 * countAPI()
 *
 *    This routineshoule be called by bemcloud when a client API is verified.
 *  input 2 parameters :
 *    INPUT: userName : bemcloud userName.
 *    INPUT: apiType  : bemcloud apiType. now support (0-9)
 *----------------------------------------------------------------------
 */
function* countAPI(userId, appID, apiType) {
  let i = 0;
  let hasAdd = 0;
  let keyName = userId + '_' + appID;

  if (false == hasInit) {
    yield initCountAPI();
    hasInit = true;
  }

  if ((parseInt(apiType) > countAPInormalData.maxApiTypeNumber) || (parseInt(apiType) < 0)) {
    //console.log("apiType only support 0-19 so far.");
    return false;
  }

  if (0 == ArrearageList[userId]) {
    /* userid is in ArrearageList. */
    //console.log("in countAPI useId "+userId+" is in ArrearageList");
    return false;
  }

  if (bufferUserObj[keyName]) {
    bufferUserObj[keyName].count += 1;
    if (bufferUserObj[keyName][apiType]) {
      bufferUserObj[keyName][apiType] += 1;
    } else {
      bufferUserObj[keyName][apiType] = 1;
    }
  } else {
    let newUser = {};
    newUser.keyName = keyName;
    newUser.count = 1;
    newUser[apiType] = 1;
    bufferUserObj[keyName] = newUser;
    console.log(bufferUserObj);
  }



  if (Object.keys(bufferUserObj).length < countAPInormalData.apiBufferUserCount) {
    return true;
  }

  sendOutAPICount();

  return true;
}

/*
 *----------------------------------------------------------------------
 *
 * sendOutAPICount()
 * called by bemcloud when init.
 *    INPUT: APIServerPort    : bemcloud apicount udp server port number.
 *    INPUT: countServerPort  : countD serverport number.
 *----------------------------------------------------------------------
 */
function sendOutAPICount() {
  let message = JSON.stringify(bufferUserObj);

  if (Object.keys(bufferUserObj).length == 0) {
    return;
  }

  console.log(message);
  client.send(message, 0, message.length, countAPInormalData.CountSDPort, "localhost", function(err, bytes) {});

  bufferUserObj = {};

  return;
}
/*
 *----------------------------------------------------------------------
 *
 * criticalChange()
 * called when critical change happen in zookeeper.
 *    INPUT: jsonData    : bemcloud apicount udp server port number.
 *    INPUT: countServerPort  : countD serverport number.
 *----------------------------------------------------------------------
 */
function countAPIcriticalChange(jsonData) {
  //console.log("criticalChange is called in countAPI and data is.");
  //console.log(jsonData);

  return;
}


function initUdpServer() {
  countApiServer = dgram.createSocket("udp4");
  countApiServer.on("message", function(msg, rinfo) {
    console.log("Server got: " + msg + " from " + rinfo.address + ":" + rinfo.port);
    /* valid package? */
    var string = new String(msg); //don't need delete?!!
    let ArrearageListTmp = string.split(" ");
    if (ArrearageListTmp.length > 0) {
      /* ArrearageListTmp[0] can simple verify the package is valid. */
      //           console.log("ArrearageListTmp ArrearageListTmp.length is: "+ArrearageListTmp.length
      //           +"content is "+ArrearageListTmp);
      if ((parseInt(ArrearageListTmp[0]) + 1) == (ArrearageListTmp.length)) {
        ArrearageList = {};
        for (let i = 1; i < ArrearageListTmp.length; i++) {
          ArrearageList[ArrearageListTmp[i]] = 0;
        }
        console.log("after reveive and parse ArrearageList is ");
        console.log(ArrearageList);
      }

    }
  });

  countApiServer.on("listening", function() {
    let address = countApiServer.address();
    //console.log("server listening "+ address.address+":"+address.port);
  });

  countApiServer.bind(countAPInormalData.coutAPIPort);

  return;
}
/*
 *----------------------------------------------------------------------
 *
 *  countAPInormalDataChange(jsonData)
 *  called when normal change happen in zookeeper.
 *    INPUT: jsonData    : changed data.
 *----------------------------------------------------------------------
 */

function countAPInormalDataChange(jsonData) {
  //    console.log("countAPInormalDataChange is called in countAPI and data is.");
  //    console.log(jsonData);

  if ((jsonData.coutAPIPort) && (jsonData.coutAPIPort != countAPInormalData.coutAPIPort)) {
    countApiServer.close();
    countAPInormalData.coutAPIPort = jsonData.coutAPIPort;
    initUdpServer();

    return;
  }

  if (jsonData.CountSDPort) {
    countAPInormalData.CountSDPort = jsonData.CountSDPort;
    return;
  }

  if (jsonData.apiBufferUserCount) {
    sendOutAPICount();
    countAPInormalData.apiBufferUserCount = jsonData.apiBufferUserCount;
    return;
  }

  if (jsonData.apiBufferTimer) {
    clearInterval(intervalId);
    countAPInormalData.apiBufferTimer = jsonData.apiBufferTimer;
    intervalId = setInterval(sendOutAPICount, countAPInormalData.apiBufferTimer * 1000);

    return;
  }

  if (jsonData.maxApiTypeNumber) {
    sendOutAPICount();
    countAPInormalData.maxApiTypeNumber = jsonData.maxApiTypeNumber;
    return;
  }

  return;
}
/*
 *----------------------------------------------------------------------
 *
 * initCountAPI()
 * called by bemcloud when init.
 *    INPUT: APIServerPort    : bemcloud apicount udp server port number.
 *    INPUT: countServerPort  : countD serverport number.
 *----------------------------------------------------------------------
 */
function* initCountAPI() {
  let moduleName = "countAPI";
  let getData;

  ArrearageList = {};
  bufferUserObj = {};

  zkAPI = new BemZk();
  zkAPI.initZookeeper(moduleName, countAPIcriticalChange, countAPInormalDataChange);
  getData = yield zkAPI.getNormalZkData(moduleName, "coutAPIPort");
  if (getData != undefined) {
    countAPInormalData.coutAPIPort = getData;
  }

  getData = yield zkAPI.getNormalZkData(moduleName, "CountSDPort");
  if (getData != undefined) {
    countAPInormalData.CountSDPort = getData;
  }

  getData = yield zkAPI.getNormalZkData(moduleName, "apiBufferUserCount");
  if (getData != undefined) {
    countAPInormalData.apiBufferUserCount = getData;
  }

  getData = yield zkAPI.getNormalZkData(moduleName, "apiBufferTimer");
  if (getData != undefined) {
    countAPInormalData.apiBufferTimer = getData;
  }

  getData = yield zkAPI.getNormalZkData(moduleName, "maxApiTypeNumber");
  if (getData != undefined) {
    countAPInormalData.maxApiTypeNumber = getData;
  }
  console.log(countAPInormalData);
  initUdpServer();

  intervalId = setInterval(sendOutAPICount, countAPInormalData.apiBufferTimer * 1000);
  //    console.log("initCountAPI finished.");
  return;
}

exports.countAPI = countAPI;
