'use strict';

let thunkify = require('thunkify');
let co = require('co');
let parallel = require('co-parallel');
let zookeeper = require('node-zookeeper-client');
let client = zookeeper.createClient('localhost:2181', {
  sessionTimeout: 30000
});
var assert = require('assert');


let db;
let dbOpen;
let countApi = require('./');
var BemZk = require('bem-zk');
var charge = require('../account/charge/charge');
let uid = "999";
let appId = "gdTamsJ4uNjvCpERiYrfgqrmkncYkUus";
let ChargeStandard = {
  "phase": 3,
  "0": {
    count: 10,
    charge: 1
  },
  "1": {
    count: 20,
    charge: 2
  },
  "2": {
    count: 30,
    charge: 3
  },
  "ladder": {
    count: 20,
    charge: 1
  }
};

let jsonDir = {
  '/Bemcloud': '1',
  '/Bemcloud/countMD': '2',
  '/Bemcloud/countSD': '3',
  '/Bemcloud/countAPI': '4',
  '/Bemcloud/accountD': '5',
  '/Bemcloud/history': '8',
  '/Bemcloud/countAPI/NormalData': '6',
  '/Bemcloud/countAPI/CirticalData': '7',
  '/Bemcloud/countAPI/NormalData/maxApiTypeNumber': '11',
  '/Bemcloud/countAPI/NormalData/coutAPIPort': '12345',
  '/Bemcloud/countAPI/NormalData/apiBufferUserCount': '200',
  '/Bemcloud/countAPI/NormalData/apiBufferTimer': '10',
  '/Bemcloud/countAPI/NormalData/CountSDPort': '12346',
  '/Bemcloud/countSD/NormalData': '6',
  '/Bemcloud/countSD/CirticalData': '7',
  '/Bemcloud/countSD/NormalData/coutAPIPort': '12345',
  '/Bemcloud/countSD/NormalData/CountSDPort': '12346',
  '/Bemcloud/countSD/NormalData/CountMDPort': '12347',
  '/Bemcloud/countSD/NormalData/CountMDIP': '127.0.0.1',
  '/Bemcloud/countSD/NormalData/BufferTime': '10',
  '/Bemcloud/countSD/NormalData/maxApiTypeNumber': '10',
  '/Bemcloud/countSD/NormalData/area': '2',
  '/Bemcloud/countMD/NormalData': '6',
  '/Bemcloud/countMD/CirticalData': '7',
  '/Bemcloud/countMD/NormalData/mongoDbIP': '127.0.0.1',
  '/Bemcloud/countMD/NormalData/mongoDbPort': '27017',
  '/Bemcloud/countMD/NormalData/CountMDPort': '12347',
  '/Bemcloud/countMD/NormalData/maxParallelNumber': '30',
  '/Bemcloud/countMD/NormalData/intervalTimer': '60',
  '/Bemcloud/countMD/NormalData/maxApiTypeNumber': '10',
  '/Bemcloud/accountD/NormalData': '6',
  '/Bemcloud/accountD/CirticalData': '7',
  '/Bemcloud/accountD/NormalData/mongoDbIP': '127.0.0.1',
  '/Bemcloud/accountD/NormalData/mongoDbPort': '27017',
  '/Bemcloud/accountD/NormalData/timerInterval': '60',
  '/Bemcloud/accountD/NormalData/maxParallelNumber': '30',
  '/Bemcloud/accountD/NormalData/maxApiTypeNumber': '10',
  '/Bemcloud/history/NormalData': '6',
  '/Bemcloud/history/CirticalData': '7',
  '/Bemcloud/history/NormalData/mongoDbIP': '127.0.0.1',
  '/Bemcloud/history/NormalData/mongoDbPort': '27017'
};
/*
 *----------------------------------------------------------------------------
 *
 *  deleteTestData
 *
 *
 *----------------------------------------------------------------------------
 */
function* initTestData() {
  client.connect();
  // console.log(module);
  // yield *countApi.initCountAPI();

  console.log("initZookeeper finished");
  let mongo = require("mongodb");
  let server = new mongo.Server('127.0.0.1', 27017, {
    auto_reconnect: true
  });

  db = new mongo.Db("_BEMCLOUD", server, {
    safe: true
  });
  dbOpen = yield thunkify(db.open).call(db);
  console.log("init DB finished");

}

/*
 *----------------------------------------------------------------------------
 *
 *  deleteTestData
 *
 *
 *----------------------------------------------------------------------------
 */

function* deleteDbData() {
  let collection = dbOpen.collection('userCountHistory');
  yield thunkify(collection.remove).call(collection, {
    "userId": "999"
  });
  collection = dbOpen.collection('userCountDailyHistory');
  yield thunkify(collection.remove).call(collection, {
    "userId": "999"
  });
  collection = dbOpen.collection('userCountMonthHistory');
  yield thunkify(collection.remove).call(collection, {
    "userId": "999"
  });
  collection = dbOpen.collection('chargeHistory');
  yield thunkify(collection.remove).call(collection, {
    "userId": "999"
  });
  console.log("after userID=999 remove data in db");

}


function* deleteTestData() {
  yield deleteDbData();

  return;
}

/*
 *----------------------------------------------------------------------------
 *
 *  createTestData
 *
 *
 *----------------------------------------------------------------------------
 */
function* createZkData() {

  console.log("in createZkData");
  for (var item in jsonDir) {
    //console.log(item);
    //console.log(jsonDir[item]);

    let data = new Buffer(jsonDir[item]);
    try {
      yield thunkify(client.create).call(client, item, data);
    } catch (err) {

    }
    //   console.log(data);
    try {
      let stat = yield thunkify(client.setData).call(client, item, data);
      //       console.log(stat);
    } catch (error) {
      console.log(error.stack);

    }


  }

  console.log("finish createZkData");


}

function* createDbData() {

  console.log("in createDbData");
  var userData = {
    "userName": "testuser",
    "area": 2,
    "count": 0,
    "userId": "999",
    "money": 2,
    "inService": 1
  };

  let userCollection = dbOpen.collection('user');
  console.log("in createDbData: dbOpen success");
  let result;
  try {
    result = yield thunkify(userCollection.update).call(userCollection, {
        "userId": '999'
      },
      userData, {
        upsert: true
      });
    console.log("createDbData: userCollection.update).call finished. ");
  } catch (error) {
    console.log(error);
  }

  let chargeStandCollection = dbOpen.collection('bemChargeStandard');
  result = yield thunkify(chargeStandCollection.remove).call(chargeStandCollection, {});
  result = yield thunkify(chargeStandCollection.update).call(chargeStandCollection, {},
    ChargeStandard, {
      upsert: true
    });
  console.log("finish createDbData.");
}

function* createTestData() {
  yield createZkData();
  console.log("after createZkData");

  yield createDbData();
}



describe('account whole test', function() {
  this.timeout(400 * 1000);
  before(function(done) {
    co(function*() {
      yield initTestData();
      yield deleteTestData();
      yield createTestData();
      client.close();
      //BemZk=new BemZk();
      // BemZk.initZookeeper(module,criticalChange,normalDataChange);
    }).then(done, done);
  });

  it('api type is bigger than maxApiNumber in service', function(done) {
    co(function*() {
      let inservice;
      inservice = yield countApi.countAPI('99999', appId, '100');
      assert.equal(false, inservice);
    }).then(done, done);
  });

  it('user in service', function(done) {
    co(function*() {
      let inservice;
      inservice = yield countApi.countAPI('99999',appId, '5');
      console.log('lx:user in service');
      console.log(inservice);
      assert.equal(true, inservice);
    }).then(done, done);;

  });

  it(' in service for user 999', function(done) {
    co(function*() {
      for (let i = 0; i < 100; i++) {
        yield countApi.countAPI('999', appId, '1');
      }
      setTimeout(function() {
        co(function*() {
          let userCollection = dbOpen.collection('user');
          let findResult;
          let inservice = 0;
          findResult = yield thunkify(userCollection.find).call(userCollection, {
            "userId": '999'
          }, {
            inService: 1,
            _id: 0
          });

          let docs = yield thunkify(findResult.toArray).call(findResult);
          console.log(docs);
          inservice = docs[0].inService
          assert.equal(1, inservice);
        }).then(done, done);

      }, 2 * 1000);
    });
  });

  it(' check db result after 3 minutes.', function(done) {
    co(function*() {
      for (let i = 0; i < 100; i++) {
        yield countApi.countAPI('999', appId, '2');
      }
      setTimeout(function() {
        co(function*() {
          let userCollection = dbOpen.collection('user');
          let findResult;
          let inservice = 0;
          findResult = yield thunkify(userCollection.find).call(userCollection, {
            "userId": '999'
          }, {
            inService: 1,
            _id: 0
          });

          let docs = yield thunkify(findResult.toArray).call(findResult);
          inservice = docs[0].inService;
          assert.equal(0, inservice);

          userCollection = dbOpen.collection('userCountDailyHistory');
          findResult = yield thunkify(userCollection.find).call(userCollection, {
            "userId": '999'
          }, {});
          docs = yield thunkify(findResult.toArray).call(findResult);
          assert.equal(200, docs[0].count);
          assert.equal(100, docs[0]['1']);
          assert.equal(100, docs[0]['2']);

          userCollection = dbOpen.collection('userCountMonthHistory');
          findResult = yield thunkify(userCollection.find).call(userCollection, {
            "userId": '999'
          }, {});
          docs = yield thunkify(findResult.toArray).call(findResult);
          assert.equal(200, docs[0].count);
          assert.equal(100, docs[0]['1']);
          assert.equal(100, docs[0]['2']);

          userCollection = dbOpen.collection('chargeHistory');
          findResult = yield thunkify(userCollection.find).call(userCollection, {
            "userId": '999'
          }, {});
          docs = yield thunkify(findResult.toArray).call(findResult);
          assert.equal(12, docs[0].charge);

        }).then(done, done);

      }, 300 * 1000);
    });
  });

  it(' out of service api check.', function(done) {
    co(function*() {
      yield countApi.countAPI('999', appId, '1');

      setTimeout(function() {
        co(function*() {
          let inservice;
          inservice = yield countApi.countAPI('999', appId, '1');
          assert.equal(false, inservice);
        }).then(done, done);;
      }, 25 * 1000);

    });
  });

});
